# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Tests for helper functions."""

from pylunar import mjd_to_date_tuple, tuple_to_string


class TestHelperFunctions:
    def test_tuple_to_string(self) -> None:
        coordinate = (-34, 23, 12)
        assert tuple_to_string(coordinate) == "-34:23:12"

    def test_mjd_to_date_tuple(self) -> None:
        date_tuple = mjd_to_date_tuple(41564.48448662116)
        assert date_tuple == (2013, 10, 18, 23, 37, 39.644068)
        date_tuple = mjd_to_date_tuple(41564.48448662116, round_off=True)
        assert date_tuple == (2013, 10, 18, 23, 37, 39)



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.



# -*- coding: utf-8 -*-
#
# complexity documentation build configuration file, created by
# sphinx-quickstart on Tue Jul  9 22:26:36 2013.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

from __future__ import annotations

import sys, os

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#sys.path.insert(0, os.path.abspath('.'))

cwd = os.getcwd()
parent = os.path.dirname(cwd)
sys.path.insert(0, parent)

import pylunar

# -- General configuration -----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.autosummary',
              'sphinx.ext.napoleon', 'sphinx.ext.viewcode']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'Python Lunar'
copyright = u'2016-2024, Michael Reuter'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = pylunar.__version__
# The full version, including alpha/beta/rc tags.
release = pylunar.__version__

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'sphinx_rtd_theme'

# Output file base name for HTML help builder.
htmlhelp_basename = 'pylunardoc'

# -- Options for LaTeX output --------------------------------------------------

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('index', 'pylunar.tex', u'Python Lunar Documentation',
   u'Michael Reuter', 'manual'),
]

# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('index', 'pylunar', u'Python Lunar Documentation',
     [u'Michael Reuter'], 1)
]

# -- Options for Texinfo output ------------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
  ('index', 'pylunar', u'Python Lunar Documentation',
   u'Michael Reuter', 'pylunar',
   'Information for completing the Astronomical League\'s Lunar and Lunar II observing programs..',
   'Miscellaneous'),
]



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Module for the LunarFeatureContainer class."""

from __future__ import annotations

__all__ = ["LunarFeatureContainer"]

import collections
import sys
from typing import Generator

if sys.version_info >= (3, 10):
    from importlib.resources import files
else:
    from importlib_resources import files

import sqlite3

from .lunar_feature import LunarFeature
from .moon_info import MoonInfo


class LunarFeatureContainer:
    """Collection of Lunar features available from the database.

    Parameters
    ----------
    club_name : str
        The name of the observing club to sort on. Values are Lunar and
        LunarII.
    """

    def __init__(self, club_name: str):
        dbname = str(files("pylunar.data").joinpath("lunar.db"))
        self.conn = sqlite3.connect(dbname)
        self.club_name = club_name
        self.features: dict[int, LunarFeature] = collections.OrderedDict()
        self.club_type: set[str] = set()
        self.feature_type: set[str] = set()

    def __iter__(self) -> Generator[LunarFeature, None, None]:
        """Create iterator for container.

        Yields
        ------
        :class:`.LunarFeature`
            The current lunar feature.
        """
        yield from self.features.values()

    def __len__(self) -> int:
        """Length of the container.

        Returns
        -------
        int
            The length of the container.
        """
        return len(self.features)

    def load(self, moon_info: MoonInfo | None = None, limit: int | None = None) -> None:
        """Read the Lunar features from the database.

        Parameters
        ----------
        moon_info : :class:`.MoonInfo`, optional
            Instance of the Lunar information class.
        limit : int, optional
            Restrict the number of features read to the given value.
        """
        if len(self.features) != 0:
            self.features = collections.OrderedDict()

        cur = self.conn.cursor()
        sql = f'select * from Features where Lunar_Code = "{self.club_name}" or ' 'Lunar_Code = "Both"'
        if limit is not None:
            sql += f" limit {limit}"
        cur.execute(sql)

        for row in cur:
            feature = LunarFeature.from_row(row)
            is_visible = True if moon_info is None else moon_info.is_visible(feature)
            if is_visible:
                self.features[id(feature)] = feature
                self.club_type.add(row[11])
                self.feature_type.add(row[7])

        cur.close()



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

from pylunar import MoonInfo
location = ((35, 58, 10), (-84, 19, 0))
obs_datetime = (2013, 10, 18, 22, 0, 0)
mi = MoonInfo(location[0], location[1])
mi.update(obs_datetime)



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Module for holding types."""

from __future__ import annotations

import sys

if sys.version_info >= (3, 10):
    from typing import TypeAlias
else:
    from typing_extensions import TypeAlias

from typing import List, Tuple, Union

DateTimeTuple: TypeAlias = Tuple[int, int, int, int, int, Union[int, float]]
MoonPhases: TypeAlias = List[Tuple[str, Union[DateTimeTuple, str]]]
DmsCoordinate: TypeAlias = Tuple[int, int, int]
Range: TypeAlias = Tuple[float, float]
LunarFeatureList: TypeAlias = Tuple[
    str, float, float, float, float, float, str, str, str, str, Union[str, None]
]
FeatureRow: TypeAlias = Tuple[int, str, float, float, float, float, float, str, str, str, str, str]



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

__all__ = [
    "__author__",
    "__email__",
    "__version__",
    "LunarFeature",
    "LunarFeatureContainer",
    "mjd_to_date_tuple",
    "MoonInfo",
    "tuple_to_string",
    "version_info",
]

from importlib.metadata import PackageNotFoundError, version

__author__ = "Michael Reuter"
__email__ = "mareuternh@gmail.com"
try:
    __version__ = version("pylunar")
except PackageNotFoundError:
    # package is not installed
    __version__ = "0.0.0"

version_info = __version__.split(".")
"""The decomposed version, split across "``.``."

Use this for version comparison.
"""

from .helpers import mjd_to_date_tuple, tuple_to_string
from .lunar_feature import LunarFeature
from .lunar_feature_container import LunarFeatureContainer
from .moon_info import MoonInfo



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Tests for the MoonInfo class."""

from pylunar import LunarFeature, MoonInfo


class TestMoonInfo:
    def setup_class(self) -> None:
        location = ((35, 58, 10), (-84, 19, 0))
        self.obs_datetime = (2013, 10, 18, 22, 0, 0)
        self.timezone = "America/New_York"
        self.mi = MoonInfo(location[0], location[1])
        self.date_list = [
            (2013, 10, 5, 0, 0, 0),
            (2013, 10, 8, 6, 0, 0),
            (2013, 10, 11, 22, 0, 0),
            (2013, 10, 12, 3, 30, 0),
            (2013, 10, 19, 1, 30, 0),
            (2013, 10, 24, 15, 0, 0),
            (2013, 10, 26, 23, 40, 0),
            (2013, 11, 2, 23, 0, 0),
        ]

    def test_basic_information_after_creation(self) -> None:
        assert self.mi.observer is not None
        assert self.mi.moon is not None

    def test_information_after_update(self) -> None:
        self.mi.update(self.obs_datetime)
        assert self.mi.observer.date == 41564.416666666664

    def test_moon_information(self) -> None:
        self.mi.update(self.obs_datetime)

        assert self.mi.age() == 13.892695999260468
        assert self.mi.fractional_age() == 0.4707676682458111
        assert self.mi.colong() == 83.97189956624061
        assert self.mi.fractional_phase() == 0.9998519924481626
        assert self.mi.phase_name() == "FULL_MOON"
        assert self.mi.libration_lon() == 5.23107551788429
        assert self.mi.libration_lat() == -1.4788210646482465
        assert self.mi.libration_phase_angle() == 105.7855572234932
        assert self.mi.altitude() == -9.814919511832146
        assert self.mi.azimuth() == 69.75156520051686
        assert self.mi.time_from_new_moon() == 333.42470398225123
        assert self.mi.time_to_new_moon() == 374.83273694326635
        assert self.mi.time_to_full_moon() == 0.0678198272944428
        assert self.mi.ra() == 23.331890450649784
        assert self.mi.dec() == 10.129795616523591
        assert self.mi.earth_distance() == 386484.25078267464
        assert self.mi.angular_size() == 0.5159071519639757
        assert self.mi.magnitude() == -12.63
        assert self.mi.subsolar_lat() == -0.3366501792590513
        assert self.mi.elongation() == 178.56298828125

        rise_set_times = self.mi.rise_set_times(self.timezone)
        position_names = [x[0] for x in rise_set_times]
        assert position_names == ["transit", "set", "rise"]
        assert rise_set_times[0][1] == (2013, 10, 18, 0, 43, 21)

        next_four_phases = self.mi.next_four_phases()
        phase_names = [x[0] for x in next_four_phases]
        assert phase_names == ["full_moon", "last_quarter", "new_moon", "first_quarter"]
        assert next_four_phases[0][1] == (2013, 10, 18, 23, 37, 39.633078)

    def test_different_elongations(self) -> None:
        self.mi.update((2013, 10, 6, 22, 0, 0))
        assert self.mi.elongation() == 23.902420043945312
        self.mi.update((2013, 10, 24, 22, 0, 0))
        assert self.mi.elongation() == 247.54827117919922
        self.mi.update((2013, 10, 31, 22, 0, 0))
        assert self.mi.elongation() == 326.54500579833984

    def test_different_rise_set_times(self) -> None:
        self.mi.update((2013, 10, 17, 22, 0, 0))
        rise_set_times = self.mi.rise_set_times(self.timezone)
        position_names = [x[0] for x in rise_set_times]
        assert position_names == ["transit", "set", "rise"]
        assert rise_set_times[0][1] == "Does not transit"
        self.mi.update((2013, 9, 26, 22, 0, 0))
        rise_set_times = self.mi.rise_set_times(self.timezone)
        position_names = [x[0] for x in rise_set_times]
        assert position_names == ["rise", "transit", "set"]
        assert rise_set_times[0][1] == "Does not rise"

    def test_state_reset_after_rise_test_call(self) -> None:
        self.mi.update(self.obs_datetime)
        self.mi.rise_set_times(self.timezone)
        assert self.mi.colong() == 83.97189956624061

    def test_different_phase_names(self) -> None:
        self.mi.update((2013, 10, 18, 18, 0, 0))
        assert self.mi.phase_name() == "WAXING_GIBBOUS"
        self.mi.update((2013, 10, 5, 0, 0, 0))
        assert self.mi.phase_name() == "NEW_MOON"
        self.mi.update((2013, 10, 8, 6, 0, 0))
        assert self.mi.phase_name() == "WAXING_CRESCENT"
        self.mi.update((2013, 10, 11, 22, 0, 0))
        assert self.mi.phase_name() == "FIRST_QUARTER"
        self.mi.update((2013, 10, 12, 3, 30, 0))
        assert self.mi.phase_name() == "WAXING_GIBBOUS"
        self.mi.update((2013, 10, 19, 1, 30, 0))
        assert self.mi.phase_name() == "FULL_MOON"
        self.mi.update((2013, 10, 24, 15, 0, 0))
        assert self.mi.phase_name() == "WANING_GIBBOUS"
        self.mi.update((2013, 10, 26, 23, 40, 0))
        assert self.mi.phase_name() == "LAST_QUARTER"
        self.mi.update((2013, 11, 2, 23, 0, 0))
        assert self.mi.phase_name() == "WANING_CRESCENT"

    def test_different_phase_emoji(self) -> None:
        self.mi.update((2013, 10, 18, 18, 0, 0))
        assert self.mi.phase_emoji() == "🌔"
        self.mi.update((2013, 10, 5, 0, 0, 0))
        assert self.mi.phase_emoji() == "🌑"
        self.mi.update((2013, 10, 8, 6, 0, 0))
        assert self.mi.phase_emoji() == "🌒"
        self.mi.update((2013, 10, 11, 22, 0, 0))
        assert self.mi.phase_emoji() == "🌓"
        self.mi.update((2013, 10, 12, 3, 30, 0))
        assert self.mi.phase_emoji() == "🌔"
        self.mi.update((2013, 10, 19, 1, 30, 0))
        assert self.mi.phase_emoji() == "🌕"
        self.mi.update((2013, 10, 24, 15, 0, 0))
        assert self.mi.phase_emoji() == "🌖"
        self.mi.update((2013, 10, 26, 23, 40, 0))
        assert self.mi.phase_emoji() == "🌗"
        self.mi.update((2013, 11, 2, 23, 0, 0))
        assert self.mi.phase_emoji() == "🌘"

    def test_colong_to_long(self) -> None:
        self.mi.update(self.date_list[0])
        assert self.mi.colong_to_long() == 85.63604081994191
        self.mi.update(self.date_list[1])
        assert self.mi.colong_to_long() == 46.01878527028475
        self.mi.update(self.date_list[2])
        assert self.mi.colong_to_long() == 1.2909929717534965
        self.mi.update(self.date_list[3])
        assert self.mi.colong_to_long() == -1.5152263359641438
        self.mi.update(self.date_list[4])
        assert self.mi.colong_to_long() == -85.74623915427537
        self.mi.update(self.date_list[5])
        assert self.mi.colong_to_long() == 26.4615811331646
        self.mi.update(self.date_list[6])
        assert self.mi.colong_to_long() == -2.1812638653489103
        self.mi.update(self.date_list[7])
        assert self.mi.colong_to_long() == -87.15293941124628

    def test_time_of_day(self) -> None:
        self.mi.update(self.date_list[0])
        assert self.mi.time_of_day() == "MORNING"
        self.mi.update(self.date_list[1])
        assert self.mi.time_of_day() == "MORNING"
        self.mi.update(self.date_list[2])
        assert self.mi.time_of_day() == "MORNING"
        self.mi.update(self.date_list[3])
        assert self.mi.time_of_day() == "MORNING"
        self.mi.update(self.date_list[4])
        assert self.mi.time_of_day() == "MORNING"
        self.mi.update(self.date_list[5])
        assert self.mi.time_of_day() == "EVENING"
        self.mi.update(self.date_list[6])
        assert self.mi.time_of_day() == "EVENING"
        self.mi.update(self.date_list[7])
        assert self.mi.time_of_day() == "EVENING"

    def test_is_visible(self) -> None:
        feature1 = LunarFeature(
            "A", 0.1, 0.0, 46.0, 0.01, 0.01, "Crater", "Taruntius", "LAC-61", "Lunar", "Telescope"
        )
        feature2 = LunarFeature(
            "B", 1.0, 50.0, 46.0, 0.5, 0.5, "Crater", "Endymion", "LAC-14", "Lunar", "Binocular"
        )
        feature3 = LunarFeature(
            "C", 100.0, -30.0, 46.0, 5.0, 10.0, "Mare", "Fracastorius", "LAC-97", "Lunar", "Naked Eye"
        )
        self.mi.update(self.date_list[0])
        assert self.mi.is_visible(feature1) is False
        assert self.mi.is_visible(feature2) is False
        assert self.mi.is_visible(feature3) is False
        self.mi.update((2013, 10, 8, 6, 15, 0))
        assert self.mi.is_visible(feature1) is True
        assert self.mi.is_visible(feature2) is False
        assert self.mi.is_visible(feature3) is False
        self.mi.update((2013, 10, 8, 7, 0, 0))
        assert self.mi.is_visible(feature1) is True
        assert self.mi.is_visible(feature2) is True
        assert self.mi.is_visible(feature3) is False
        self.mi.update((2013, 10, 8, 16, 0, 0))
        assert self.mi.is_visible(feature1) is True
        assert self.mi.is_visible(feature2) is True
        assert self.mi.is_visible(feature3) is True
        self.mi.update((2013, 10, 9, 12, 0, 0))
        assert self.mi.is_visible(feature1) is False
        assert self.mi.is_visible(feature2) is True
        assert self.mi.is_visible(feature3) is True
        self.mi.update((2013, 10, 10, 5, 0, 0))
        assert self.mi.is_visible(feature1) is False
        assert self.mi.is_visible(feature2) is False
        assert self.mi.is_visible(feature3) is True
        self.mi.update((2013, 10, 21, 2, 30, 0))
        assert self.mi.is_visible(feature1) is False
        assert self.mi.is_visible(feature2) is True
        assert self.mi.is_visible(feature3) is True
        self.mi.update((2013, 10, 21, 19, 0, 0))
        assert self.mi.is_visible(feature1) is True
        assert self.mi.is_visible(feature2) is True
        assert self.mi.is_visible(feature3) is True
        self.mi.update((2013, 10, 22, 15, 0, 0))
        assert self.mi.is_visible(feature1) is True
        assert self.mi.is_visible(feature2) is True
        assert self.mi.is_visible(feature3) is False
        self.mi.update((2013, 10, 23, 0, 0, 0))
        assert self.mi.is_visible(feature1) is True
        assert self.mi.is_visible(feature2) is False
        assert self.mi.is_visible(feature3) is False
        self.mi.update((2013, 10, 23, 1, 0, 0))
        assert self.mi.is_visible(feature1) is False
        assert self.mi.is_visible(feature2) is False
        assert self.mi.is_visible(feature3) is False

    def test_is_libration_ok(self) -> None:
        feature1 = LunarFeature(
            "A", 374.0, -2.0, 87.0, 12.0, 12.0, "Mare", "Ansgarius", "LAC-81", "LunarII", None
        )
        feature2 = LunarFeature("B", 358.0, 13.0, 86.5, 9.0, 12.0, "Mare", "Neper", "LAC-63", "LunarII", None)
        feature3 = LunarFeature(
            "C", 682.0, -19.5, -95.0, 22.5, 23.0, "Mons", "Mare Orientale", "LAC-108", "LunarII", None
        )
        self.mi.update((2017, 5, 27, 12, 21, 0))
        assert self.mi.is_libration_ok(feature1) is True
        assert self.mi.is_libration_ok(feature2) is True
        assert self.mi.is_libration_ok(feature3) is False
        assert self.mi.is_visible(feature1) is True
        self.mi.update((2017, 11, 24, 22, 0, 0))
        assert self.mi.is_libration_ok(feature1) is False
        assert self.mi.is_libration_ok(feature2) is False
        assert self.mi.is_visible(feature2) is False
        assert self.mi.is_libration_ok(feature3) is True
        assert self.mi.is_visible(feature3) is False
        self.mi.update((2017, 7, 17, 6, 0, 0))
        assert self.mi.is_libration_ok(feature1) is False
        assert self.mi.is_libration_ok(feature2) is False
        assert self.mi.is_libration_ok(feature3) is True



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Tests for the LunarFeatureContainer class."""

from pylunar import LunarFeatureContainer, MoonInfo


class TestLunarFeatureContainer:
    def setup_class(self) -> None:
        self.lc_lfc = LunarFeatureContainer("Lunar")
        self.lc2_lfc = LunarFeatureContainer("LunarII")

    def test_basic_information_after_creation(self) -> None:
        assert len(self.lc_lfc) == 0
        assert len(self.lc2_lfc) == 0

    def test_short_load(self) -> None:
        self.lc_lfc.load(limit=10)
        assert len(self.lc_lfc) == 10
        assert len(self.lc_lfc.club_type) == 2
        assert len(self.lc_lfc.feature_type) == 4

        self.lc2_lfc.load(limit=10)
        assert len(self.lc2_lfc) == 10
        assert len(self.lc2_lfc.club_type) == 2
        assert len(self.lc2_lfc.feature_type) == 5

    def test_iterator(self) -> None:
        self.lc_lfc.load(limit=2)
        assert len(self.lc_lfc) == 2
        ilfc = iter(self.lc_lfc)
        feature = next(ilfc)
        assert feature.name == "Vallis Alpes"

        self.lc2_lfc.load(limit=2)
        assert len(self.lc2_lfc) == 2
        ilfc2 = iter(self.lc2_lfc)
        feature2 = next(ilfc2)
        assert feature2.name == "Montes Jura"

    def test_visibility(self) -> None:
        location = ((35, 58, 10), (-84, 19, 0))
        mi = MoonInfo(location[0], location[1])
        mi.update((2013, 10, 12, 18, 0, 0))

        # Need to create new containers.
        lc_lfc = LunarFeatureContainer("Lunar")
        lc2_lfc = LunarFeatureContainer("LunarII")

        lc_lfc.load(mi, limit=10)
        assert len(lc_lfc) == 6
        assert len(lc_lfc.club_type) == 2
        assert len(lc_lfc.feature_type) == 4
        ilfc = iter(lc_lfc)
        feature = next(ilfc)
        assert feature.name == "Vallis Alpes"

        lc2_lfc.load(mi, limit=10)
        assert len(lc2_lfc) == 2
        assert len(lc2_lfc.club_type) == 2
        assert len(lc2_lfc.feature_type) == 2
        ilfc2 = iter(lc2_lfc)
        feature2 = next(ilfc2)
        assert feature2.name == "Vallis Alpes"



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Tests for the LunarFeature class."""

from pylunar import LunarFeature


class TestLunarFeature:
    def setup_class(self) -> None:
        self.feature_info = (
            "Clavius",
            230.77,
            -58.622830884248,
            -14.727490317559,
            7.61545753479,
            14.614604949952,
            "Crater",
            "Clavius",
            "LAC-126",
            "Both",
            "Binocular",
        )

    def test_basic_information_after_creation(self) -> None:
        lf = LunarFeature(*self.feature_info)
        assert lf.name == self.feature_info[0]
        assert lf.diameter == self.feature_info[1]
        assert lf.latitude == self.feature_info[2]
        assert lf.longitude == self.feature_info[3]
        assert lf.delta_latitude == self.feature_info[4]
        assert lf.delta_longitude == self.feature_info[5]
        assert lf.feature_type == self.feature_info[6]
        assert lf.quad_name == self.feature_info[7]
        assert lf.quad_code == self.feature_info[8]
        assert lf.code_name == self.feature_info[9]
        assert lf.lunar_club_type == self.feature_info[10]
        assert lf.latitude_range() == (-62.430559651643, -54.815102116853005)
        assert lf.longitude_range() == (-22.034792792535, -7.420187842583001)
        assert lf.feature_angle() == 194.10225514559056

        val = str(lf)
        assert val.startswith("Name")

    def test_creation_from_database_row(self) -> None:
        feature_row = (
            33,
            "Clavius",
            230.77,
            -58.622830884248,
            -14.727490317559,
            7.61545753479,
            14.614604949952,
            "Crater",
            "Clavius",
            "LAC-126",
            "Both",
            "Binocular",
        )
        lf = LunarFeature.from_row(feature_row)
        assert lf.name == feature_row[1]
        assert lf.diameter == feature_row[2]
        assert lf.latitude == feature_row[3]
        assert lf.longitude == feature_row[4]
        assert lf.delta_latitude == feature_row[5]
        assert lf.delta_longitude == feature_row[6]
        assert lf.feature_type == feature_row[7]
        assert lf.quad_name == feature_row[8]
        assert lf.quad_code == feature_row[9]
        assert lf.code_name == feature_row[10]
        assert lf.lunar_club_type == feature_row[11]

    def test_list_from_feature(self) -> None:
        lf = LunarFeature(*self.feature_info)
        feature_list = lf.list_from_feature()
        for value, truth in zip(feature_list, self.feature_info):
            assert value == truth



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Module for the MoonInfo class."""

from __future__ import annotations

__all__ = ["MoonInfo"]

from datetime import datetime
from enum import Enum
import math
from operator import itemgetter

import ephem
import pytz

from .helpers import mjd_to_date_tuple, tuple_to_string
from .lunar_feature import LunarFeature
from .types import DateTimeTuple, DmsCoordinate, MoonPhases


class PhaseName(Enum):
    """Phase names for the lunar cycle."""

    NEW_MOON = 0
    WAXING_CRESCENT = 1
    FIRST_QUARTER = 2
    WAXING_GIBBOUS = 3
    FULL_MOON = 4
    WANING_GIBBOUS = 5
    LAST_QUARTER = 6
    WANING_CRESCENT = 7


class TimeOfDay(Enum):
    """Time of day from the lunar terminator."""

    MORNING = 0
    EVENING = 1


class MoonInfo:
    """Handle all moon information.

    Attributes
    ----------
    observer : ephem.Observer instance.
        The instance containing the observer's location information.
    moon : ephem.Moon instance
        The instance of the moon object.

    Parameters
    ----------
    latitude : tuple of 3 ints
        The latitude of the observer in GPS DMS(Degrees, Minutes and
        Seconds) format.
    longitude : tuple of 3 ints
        The longitude of the observer in GPS DMS(Degrees, Minutes and
        Seconds) format.
    name : str, optional
        A name for the observer's location.
    """

    DAYS_TO_HOURS = 24.0
    MAIN_PHASE_CUTOFF = 2.0
    # Time cutoff (hours) around the NM, FQ, FM, and LQ phases
    FEATURE_CUTOFF = 15.0
    # The offset (degrees) from the colongitude used for visibility check
    NO_CUTOFF_TYPE = ("Landing Site", "Mare", "Oceanus")
    # Feature types that are not subject to longitude cutoffs
    LIBRATION_ZONE = 80.0
    # Latitude and/or longitude where librations have a big effect
    MAXIMUM_LIBRATION_PHASE_ANGLE_CUTOFF = 65.0
    # The maximum value of the libration phase angle difference for a feature

    reverse_phase_lookup = {
        "new_moon": (ephem.previous_last_quarter_moon, "last_quarter"),
        "first_quarter": (ephem.previous_new_moon, "new_moon"),
        "full_moon": (ephem.previous_first_quarter_moon, "first_quarter"),
        "last_quarter": (ephem.previous_full_moon, "full_moon"),
    }

    def __init__(self, latitude: DmsCoordinate, longitude: DmsCoordinate, name: str | None = None):
        self.observer = ephem.Observer()
        self.observer.lat = tuple_to_string(latitude)
        self.observer.long = tuple_to_string(longitude)
        self.moon = ephem.Moon()

    def age(self) -> float:
        """Lunar age in days.

        Returns
        -------
        float
            The lunar age.
        """
        prev_new = ephem.previous_new_moon(self.observer.date)
        return float(self.observer.date - prev_new)

    def fractional_age(self) -> float:
        """Lunar fractional age which is always less than 1.0.

        Returns
        -------
        float
            The fractional lunar age.
        """
        prev_new = ephem.previous_new_moon(self.observer.date)
        next_new = ephem.next_new_moon(self.observer.date)
        return float((self.observer.date - prev_new) / (next_new - prev_new))

    def altitude(self) -> float:
        """Lunar altitude in degrees.

        Returns
        -------
        float
            The lunar altitiude.
        """
        return math.degrees(self.moon.alt)

    def angular_size(self) -> float:
        """Lunar current angular size in degrees.

        Returns
        -------
        float
            The lunar angular size.
        """
        moon_size: float = self.moon.size
        return moon_size / 3600.0

    def azimuth(self) -> float:
        """Lunar azimuth in degrees.

        Returns
        -------
        float
            The lunar azimuth.
        """
        return math.degrees(self.moon.az)

    def colong(self) -> float:
        """Lunar selenographic colongitude in degrees.

        Returns
        -------
        float
            The lunar seleographic colongitude.
        """
        return math.degrees(self.moon.colong)

    def dec(self) -> float:
        """Lunar current declination in degrees.

        Returns
        -------
        float
            The lunar declination.
        """
        return math.degrees(self.moon.dec)

    def earth_distance(self) -> float:
        """Lunar current distance from the earth in km.

        Returns
        -------
        float
            THe earth-moon distance.
        """
        return float(self.moon.earth_distance * ephem.meters_per_au / 1000.0)

    def elongation(self) -> float:
        """Lunar elongation from the sun in degrees.

        Returns
        -------
        float
            The lunar solar elongation.
        """
        elongation = math.degrees(self.moon.elong)
        if elongation < 0:
            elongation += 360.0
        return elongation

    def fractional_phase(self) -> float:
        """Lunar fractional illumination which is always less than 1.0.

        Returns
        -------
        float
            The lunar fractional phase.
        """
        return float(self.moon.moon_phase)

    def libration_lat(self) -> float:
        """Lunar current latitudinal libration in degrees.

        Returns
        -------
        float
            The lunar libration latitude.
        """
        return math.degrees(self.moon.libration_lat)

    def libration_lon(self) -> float:
        """Lunar current longitudinal libration in degrees.

        Returns
        -------
        float
            The lunar libration longitude.
        """
        return math.degrees(self.moon.libration_long)

    def libration_phase_angle(self) -> float:
        """Phase angle of lunar current libration in degrees.

        Returns
        -------
        float
            The lunar libration phase angle.
        """
        phase_angle = math.atan2(self.moon.libration_long, self.moon.libration_lat)
        phase_angle += 2.0 * math.pi if phase_angle < 0 else 0.0
        return math.degrees(phase_angle)

    def magnitude(self) -> float:
        """Lunar current magnitude.

        Returns
        -------
        float
            The lunar magnitude.
        """
        return float(self.moon.mag)

    def colong_to_long(self) -> float:
        """Selenographic longitude in degrees based on the terminator.

        Returns
        -------
        float
            The lunar seleographic longitude.
        """
        colong: float = self.colong()
        if 90.0 <= colong < 270.0:
            longitude = 180.0 - colong
        elif 270.0 <= colong < 360.0:
            longitude = 360.0 - colong
        else:
            longitude = -colong

        return longitude

    def is_libration_ok(self, feature: LunarFeature) -> bool:
        """Determine if lunar feature is visible due to libration effect.

        Parameters
        ----------
        feature : :class:`.LunarFeature`
            The lunar feature instance to check.

        Returns
        -------
        bool
            True if visible, False if not.
        """
        is_lon_in_zone = math.fabs(feature.longitude) > self.LIBRATION_ZONE
        is_lat_in_zone = math.fabs(feature.latitude) > self.LIBRATION_ZONE
        if is_lat_in_zone or is_lon_in_zone:
            feature_angle = feature.feature_angle()
            libration_phase_angle = self.libration_phase_angle()
            delta_phase_angle = libration_phase_angle - feature_angle
            delta_phase_angle -= 360.0 if delta_phase_angle > 180.0 else 0.0

            return math.fabs(delta_phase_angle) <= self.MAXIMUM_LIBRATION_PHASE_ANGLE_CUTOFF

        return True

    def is_visible(self, feature: LunarFeature) -> bool:
        """Determine if lunar feature is visible.

        Parameters
        ----------
        feature : :class:`.LunarFeature`
            The lunar feature instance to check.

        Returns
        -------
        bool
            True if visible, False if not.
        """
        selco_lon = self.colong_to_long()
        current_tod = self.time_of_day()

        min_lon = feature.longitude - feature.delta_longitude / 2
        max_lon = feature.longitude + feature.delta_longitude / 2

        if min_lon > max_lon:
            min_lon, max_lon = max_lon, min_lon

        is_visible = False
        latitude_scaling = math.cos(math.radians(feature.latitude))
        if feature.feature_type not in MoonInfo.NO_CUTOFF_TYPE:
            cutoff = MoonInfo.FEATURE_CUTOFF / latitude_scaling
        else:
            cutoff = MoonInfo.FEATURE_CUTOFF

        if current_tod == TimeOfDay.MORNING.name:
            # Minimum longitude for morning visibility
            lon_cutoff = min_lon - cutoff
            if feature.feature_type in MoonInfo.NO_CUTOFF_TYPE:
                is_visible = selco_lon <= min_lon
            else:
                is_visible = lon_cutoff <= selco_lon <= min_lon
        else:
            # Maximum longitude for evening visibility
            lon_cutoff = max_lon + cutoff
            if feature.feature_type in MoonInfo.NO_CUTOFF_TYPE:
                is_visible = max_lon <= selco_lon
            else:
                is_visible = max_lon <= selco_lon <= lon_cutoff

        return is_visible and self.is_libration_ok(feature)

    def next_four_phases(self) -> MoonPhases:
        """Next four phases in date sorted order (closest phase first).

        Returns
        -------
        list[(str, float)]
            Set of lunar phases specified by an abbreviated phase name and
            Modified Julian Date.
        """
        phases = {}
        phases["new_moon"] = ephem.next_new_moon(self.observer.date)
        phases["first_quarter"] = ephem.next_first_quarter_moon(self.observer.date)
        phases["full_moon"] = ephem.next_full_moon(self.observer.date)
        phases["last_quarter"] = ephem.next_last_quarter_moon(self.observer.date)

        sorted_phases = sorted(phases.items(), key=itemgetter(1))
        sorted_phases = [(phase[0], mjd_to_date_tuple(phase[1])) for phase in sorted_phases]

        return sorted_phases

    def phase_name(self) -> str:
        """Return standard name of lunar phase, i.e. Waxing Cresent.

        This function returns a standard name for lunar phase based on the
        current selenographic colongitude.

        Returns
        -------
        str
            The lunar phase name.
        """
        next_phase_name = self.next_four_phases()[0][0]
        try:
            next_phase_time = getattr(ephem, f"next_{next_phase_name}")(self.observer.date)
        except AttributeError:
            next_phase_time = getattr(ephem, f"next_{next_phase_name}_moon")(self.observer.date)
        previous_phase = self.reverse_phase_lookup[next_phase_name]
        time_to_next_phase = math.fabs(next_phase_time - self.observer.date) * self.DAYS_TO_HOURS
        time_to_previous_phase = (
            math.fabs(self.observer.date - previous_phase[0](self.observer.date)) * self.DAYS_TO_HOURS
        )
        previous_phase_name = previous_phase[1]

        phase_name = ""
        if time_to_previous_phase < self.MAIN_PHASE_CUTOFF:
            phase_name = getattr(PhaseName, previous_phase_name.upper()).name
        elif time_to_next_phase < self.MAIN_PHASE_CUTOFF:
            phase_name = getattr(PhaseName, next_phase_name.upper()).name
        else:
            if previous_phase_name == "new_moon" and next_phase_name == "first_quarter":
                phase_name = PhaseName.WAXING_CRESCENT.name
            elif previous_phase_name == "first_quarter" and next_phase_name == "full_moon":
                phase_name = PhaseName.WAXING_GIBBOUS.name
            elif previous_phase_name == "full_moon" and next_phase_name == "last_quarter":
                phase_name = PhaseName.WANING_GIBBOUS.name
            elif previous_phase_name == "last_quarter" and next_phase_name == "new_moon":
                phase_name = PhaseName.WANING_CRESCENT.name
        return phase_name

    def phase_shape_in_ascii(self) -> str:
        """Display lunar phase shape in ASCII art.

        This function returns a multi-line string demonstrate current lunar
        shape in ASCII format.

        Returns
        -------
        str
            The lunar phase shape.
        """
        phase = self.phase_name()

        if phase == PhaseName.NEW_MOON.name:
            return """   _..._
 .:::::::.
:::::::::::
:::::::::::
`:::::::::'
  `':::''        """
        elif phase == PhaseName.WAXING_CRESCENT.name:
            return """   _..._
 .::::. `.
:::::::.  :
::::::::  :
`::::::' .'
  `'::'-'        """
        elif phase == PhaseName.FIRST_QUARTER.name:
            return """   _..._
 .::::  `.
::::::    :
::::::    :
`:::::   .'
  `'::.-'        """
        elif phase == PhaseName.WAXING_GIBBOUS.name:
            return """   _..._
 .::'   `.
:::       :
:::       :
`::.     .'
  `':..-'        """
        elif phase == PhaseName.FULL_MOON.name:
            return """   _..._
 .'     `.
:         :
:         :
`.       .'
  `-...-'        """
        elif phase == PhaseName.WANING_GIBBOUS.name:
            return """   _..._
 .'   `::.
:       :::
:       :::
`.     .::'
  `-..:''        """
        elif phase == PhaseName.LAST_QUARTER.name:
            return """   _..._
 .'  ::::.
:    ::::::
:    ::::::
`.   :::::'
  `-.::''        """
        elif phase == PhaseName.WAXING_CRESCENT.name:
            return """   _..._
 .' .::::.
:  ::::::::
:  ::::::::
`. '::::::'
  `-.::''        """
        else:
            return phase

    def phase_emoji(self) -> str:
        """Return standard emoji of lunar phase, i.e. '🌒'.

        This function returns a standard emoji for lunar phase based on the
        current selenographic colongitude.

        Returns
        -------
        str
            The lunar phase emoji.
        """
        return {
            "NEW_MOON": "🌑",
            "WAXING_CRESCENT": "🌒",
            "FIRST_QUARTER": "🌓",
            "WAXING_GIBBOUS": "🌔",
            "FULL_MOON": "🌕",
            "WANING_GIBBOUS": "🌖",
            "LAST_QUARTER": "🌗",
            "WANING_CRESCENT": "🌘",
        }[self.phase_name()]

    def ra(self) -> float:
        """Lunar current right ascension in degrees.

        Returns
        -------
        float
            The lunar right ascension.
        """
        return math.degrees(self.moon.ra)

    def rise_set_times(self, timezone: str) -> MoonPhases:
        """Calculate the rise, set and transit times in the local time system.

        Parameters
        ----------
        timezone : str
            The timezone identifier for the calculations.

        Returns
        -------
        list[(str, tuple)]
            Set of rise, set, and transit times in the local time system. If
            event does not happen, 'Does not xxx' is tuple value.
        """
        utc = pytz.utc
        try:
            tz = pytz.timezone(timezone)
        except pytz.UnknownTimeZoneError:
            tz = utc

        func_map = {"rise": "rising", "transit": "transit", "set": "setting"}

        # Need to set observer's horizon and pressure to get times
        old_pressure = self.observer.pressure
        old_horizon = self.observer.horizon

        self.observer.pressure = 0
        self.observer.horizon = "-0:34"

        current_date_utc = datetime(*mjd_to_date_tuple(self.observer.date, round_off=True), tzinfo=utc)  # type: ignore
        current_date = current_date_utc.astimezone(tz)
        current_day = current_date.day
        times = {}
        does_not = None
        for time_type in ("rise", "transit", "set"):
            mjd_time = getattr(self.observer, "{}_{}".format("next", func_map[time_type]))(self.moon)
            utc_time = datetime(*mjd_to_date_tuple(mjd_time, round_off=True), tzinfo=utc)  # type: ignore
            local_date = utc_time.astimezone(tz)
            if local_date.day == current_day:
                times[time_type] = local_date
            else:
                mjd_time = getattr(self.observer, "{}_{}".format("previous", func_map[time_type]))(self.moon)
                utc_time = datetime(*mjd_to_date_tuple(mjd_time, round_off=True), tzinfo=utc)  # type: ignore
                local_date = utc_time.astimezone(tz)
                if local_date.day == current_day:
                    times[time_type] = local_date
                else:
                    does_not = (time_type, f"Does not {time_type}")

        # Return observer and moon to previous state
        self.observer.pressure = old_pressure
        self.observer.horizon = old_horizon
        self.moon.compute(self.observer)

        original_sorted_times = sorted(times.items(), key=itemgetter(1))
        sorted_times: MoonPhases = [(xtime[0], xtime[1].timetuple()[:6]) for xtime in original_sorted_times]
        if does_not is not None:
            sorted_times.insert(0, does_not)

        return sorted_times

    def subsolar_lat(self) -> float:
        """Latitude in degress on the moon where the sun is overhead.

        Returns
        -------
        float
            The lunar subsolar latitude.
        """
        return math.degrees(self.moon.subsolar_lat)

    def time_of_day(self) -> str:
        """Terminator time of day.

        This function determines if the terminator is sunrise (morning) or
        sunset (evening).

        Returns
        -------
        str
            The lunar time of day.
        """
        colong = self.colong()
        if 90.0 <= colong < 270.0:
            return TimeOfDay.EVENING.name
        else:
            return TimeOfDay.MORNING.name

    def time_from_new_moon(self) -> float:
        """Time (hours) from the previous new moon.

        This function calculates the time from the previous new moon.

        Returns
        -------
        float
            The time from new moon.
        """
        previous_new_moon = ephem.previous_new_moon(self.observer.date)
        return float(MoonInfo.DAYS_TO_HOURS * (self.observer.date - previous_new_moon))

    def time_to_full_moon(self) -> float:
        """Time (days) to the next full moon.

        This function calculates the time to the next full moon.

        Returns
        -------
        float
            The time to full moon.
        """
        next_full_moon = ephem.next_full_moon(self.observer.date)
        return float(next_full_moon - self.observer.date)

    def time_to_new_moon(self) -> float:
        """Time (hours) to the next new moon.

        This function calculates the time to the next new moon.

        Returns
        -------
        float
            The time to new moon.
        """
        next_new_moon = ephem.next_new_moon(self.observer.date)
        return float(MoonInfo.DAYS_TO_HOURS * (next_new_moon - self.observer.date))

    def update(self, datetime: DateTimeTuple) -> None:
        """Update the moon information based on time.

        This fuction updates the Observer instance's datetime setting. The
        incoming datetime tuple should be in UTC with the following placement
        of values: (YYYY, m, d, H, M, S) as defined below::

            YYYY
                Four digit year

            m
                month (1-12)

            d
                day (1-31)

            H
                hours (0-23)

            M
                minutes (0-59)

            S
                seconds (0-59)

        Parameters
        ----------
        datetime : tuple
            The current UTC time in a tuple of numbers.
        """
        self.observer.date = datetime
        self.moon.compute(self.observer)



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Module for the LunarFeature class."""

from __future__ import annotations

__all__ = ["LunarFeature"]

import math
import os

from .types import FeatureRow, Range


class LunarFeature:
    """Class keeping all the information for a given Lunar feature.

    Parameters
    ----------
    name : str
        The name of the Lunar feature (no unicode).
    diameter : float
        The diameter (km) of the Lunar feature.
    latitude : float
        The latitude (degrees) of the Lunar feature. Negative is South,
        positive is North.
    longitude : float
        The longitude (degrees) of the Lunar feature. Negative is West,
        positive is East.
    delta_latitude : float
        The size (degrees) in latitude of the Lunar feature.
    delta_longitude : float
        The size (degrees) in longitude of the Lunar feature.
    feature_type : str
        The classification of the Lunar feature: i.e. Crater, Mons.
    quad_name : str
        Name of lunar quadrant containing feature's center point as
        determined by the International Astronomical Union (IAU) Working
        Group for Planetary System Nomenclature (WGPSN).
    quad_code : str
        Specific lunar quadrant containing feature's center point as
        determined by the IAU WGPSN.
    code_name : str
        The AstroLeague club name for the Lunar feature. Can be: Lunar,
        LunarII or Both.
    lunar_club_type : str or None
        The Lunar Club classification of the feature: Naked Eye, Binocular,
        Telescope. For a LunarII only feature this is None.
    """

    def __init__(
        self,
        name: str,
        diameter: float,
        latitude: float,
        longitude: float,
        delta_latitude: float,
        delta_longitude: float,
        feature_type: str,
        quad_name: str,
        quad_code: str,
        code_name: str,
        lunar_club_type: str | None,
    ):
        self.name = name
        self.diameter = diameter
        self.latitude = latitude
        self.longitude = longitude
        self.delta_latitude = delta_latitude
        self.delta_longitude = delta_longitude
        self.feature_type = feature_type
        self.quad_name = quad_name
        self.quad_code = quad_code
        self.code_name = code_name
        self.lunar_club_type = str(lunar_club_type)

    def __str__(self) -> str:
        """Class string representation.

        Returns
        -------
        str
            The string representation.
        """
        result = []
        result.append(f"Name = {self.name}")
        result.append(f"Lat/Long = ({self.latitude:.2f}, {self.longitude:.2f})")
        result.append(f"Type = {self.feature_type}")
        result.append(f"Delta Lat/Long = ({self.delta_latitude:.2f}, {self.delta_longitude:.2f})")
        return os.linesep.join(result)

    @classmethod
    def from_row(cls: type[LunarFeature], row: FeatureRow) -> LunarFeature:
        """Initialize from a database row.

        Parameters
        ----------
        row : list
            The database row containing the information.

        Returns
        -------
        :class:`.LunarFeature`
            Class initialized from database row.
        """
        return cls(*row[1:])

    def feature_angle(self) -> float:
        """Get the angle of the feature on the lunar face relative to North.

        The feature angle is determined by calculating atan2(lon, lat) and
        then adding 360 degrees if the result is less than zero. This makes
        North zero degrees, East 90 degrees, South 180 degrees and West 270
        degrees.

        Returns
        -------
        float
            The feature angle in degrees.
        """
        lat_rad = math.radians(self.latitude)
        lon_rad = math.radians(self.longitude)
        fa = math.degrees(math.atan2(lon_rad, lat_rad))
        fa += 360.0 if fa < 0 else 0.0
        return fa

    def latitude_range(self) -> Range:
        """Get the latitude range of the feature.

        Returns
        -------
        tuple(float, float)
            The (minimum, maximum) latitude values for the feature.
        """
        min_lat = self.latitude - (self.delta_latitude / 2.0)
        max_lat = self.latitude + (self.delta_latitude / 2.0)
        return (min_lat, max_lat)

    def list_from_feature(self) -> list[object]:
        """Convert the feature information into a list.

        Returns
        -------
        list
            The list of lunar features.
        """
        return [
            self.name,
            self.diameter,
            self.latitude,
            self.longitude,
            self.delta_latitude,
            self.delta_longitude,
            self.feature_type,
            self.quad_name,
            self.quad_code,
            self.code_name,
            self.lunar_club_type,
        ]

    def longitude_range(self) -> Range:
        """Get the longitude range of the feature.

        Returns
        -------
        tuple(float, float)
            The (minimum, maximum) longitude values for the feature.
        """
        min_lon = self.longitude - (self.delta_longitude / 2.0)
        max_lon = self.longitude + (self.delta_longitude / 2.0)
        return (min_lon, max_lon)



# This file is part of pylunar.
#
# Developed by Michael Reuter.
#
# See the LICENSE file at the top-level directory of this distribution
# for details of code ownership.
#
# Use of this source code is governed by a 3-clause BSD-style
# license that can be found in the LICENSE file.

"""Module for helper functions."""

from __future__ import annotations

__all__ = ["mjd_to_date_tuple", "tuple_to_string"]

import ephem

from .types import DateTimeTuple, DmsCoordinate


def mjd_to_date_tuple(mjd: float, round_off: bool = False) -> DateTimeTuple:
    """Convert a Modified Julian date to a UTC time tuple.

    Parameters
    ----------
    mjd : float
        The Modified Julian Date to convert.
    round_off : bool, optional
        Flag to round the seconds.

    Returns
    -------
    tuple
        The UTC time for the MJD.
    """
    date_tuple: DateTimeTuple
    date_tuple = tuple(int(x) for x in ephem.Date(mjd).tuple()) if round_off else ephem.Date(mjd).tuple()
    return date_tuple


def tuple_to_string(coord: DmsCoordinate) -> str:
    """Return a colon-delimited string.

    Parameters
    ----------
    coord : tuple of 3 ints
        The coordinate to transform.

    Returns
    -------
    str
        The colon-delimited coordinate string.
    """
    return ":".join([str(x) for x in coord])
